import itertools
from collections.abc import Iterable
from pathlib import Path
from typing import Union

from komposer.types import docker_compose, kubernetes
from komposer.types.cli import Context
from komposer.utils import command_to_args, to_kubernetes_name


def generate_container_environment_from_environment(
    environment: docker_compose.Environment,
) -> Iterable[kubernetes.EnvironmentVariable]:
    env_vars_iter = (
        kubernetes.EnvironmentVariable.from_string(env_var) for env_var in environment
    )
    env_vars = itertools.chain(*env_vars_iter)

    return env_vars


def generate_container_environment_from_env_file(
    context: Context, env_file: Path
) -> Iterable[kubernetes.ConfigMapEnvironmentVariable]:
    env_vars = kubernetes.ConfigMapEnvironmentVariable.from_env_file(
        context.manifest_prefix, context.docker_compose_path.parent, env_file
    )

    return env_vars


def generate_container_environment(
    context: Context, service: docker_compose.Service
) -> Iterable[Union[kubernetes.EnvironmentVariable, kubernetes.ConfigMapEnvironmentVariable]]:
    if service.environment:
        return generate_container_environment_from_environment(service.environment)

    if service.env_file:
        return generate_container_environment_from_env_file(context, service.env_file)

    return []


def generate_containter_ports(ports: list[str]) -> list[kubernetes.ContainerPort]:
    return [kubernetes.ContainerPort.from_string(port) for port in ports]


def generate_container(
    context: Context, service_name: str, service: docker_compose.Service
) -> kubernetes.Container:
    return kubernetes.Container(
        image=service.image or context.default_image,
        name=to_kubernetes_name(service_name),
        args=command_to_args(service.command),
        env=list(generate_container_environment(context, service)),
        ports=generate_containter_ports(service.ports),
    )


def generate_containers(
    context: Context, services: docker_compose.Services
) -> list[kubernetes.Container]:
    return [
        generate_container(context, service_name, service)
        for service_name, service in services.items()
    ]



from collections.abc import Sequence
from pathlib import Path
from typing import Optional

import click

from komposer.core.base import generate_manifest_from_docker_compose
from komposer.core.env_vars import replace_komposer_env_variables
from komposer.types.cli import Context, DeploymentContext, IngressContext
from komposer.utils import dump_yaml

DEFAULT_DOCKER_COMPOSE_FILENAME = Path("docker-compose.yml")
DEFAULT_DOCKER_IMAGE = "${IMAGE}"
DEFAULT_INGRESS_DOMAIN = "svc.cluster.local"


def output_raw_manifest(context: Context, manifest: dict) -> None:
    # Convert manifest to string
    output = dump_yaml(manifest)

    # Render KOMPOSER_* vars
    output = replace_komposer_env_variables(context, output)

    # Output manifest
    print(output)


@click.command()
@click.option(
    "--compose-file",
    "-f",
    default=DEFAULT_DOCKER_COMPOSE_FILENAME,
    type=click.Path(file_okay=True, dir_okay=False, resolve_path=True, path_type=Path),
    help=f"Path to the Docker compose file. Default is {DEFAULT_DOCKER_COMPOSE_FILENAME}.",
)
@click.option("--project-name", "-p", help="Name of the project")
@click.option(
    "--repository-name", "-r", required=True, help="Name of the repository of this project"
)
@click.option("--branch-name", "-b", required=True, help="Name of the branch going to be deployed")
@click.option(
    "--ingress-for-service", "-i", help="Name of the service to have an ingress associated to it"
)
@click.option(
    "--extra-manifest",
    type=click.Path(file_okay=True, dir_okay=False, resolve_path=True, path_type=Path),
    multiple=True,
    help=(
        "Path to an extra manifest file with custom items to be merged. "
        "The manifest must be in a format where the root element is a Kubernetes List node. "
        "The branch and repository labels will be added to each item; "
        "each item's name will be prefixed with the computed prefix of the generated manifest"
    ),
)
@click.option(
    "--default-image",
    default=DEFAULT_DOCKER_IMAGE,
    help=(
        "Default image to be used for those Docker Compose services whom don't have the image set. "  # noqa: E501
        f"Default is {DEFAULT_DOCKER_IMAGE}"
    ),
)
@click.option(
    "--ingress-tls-file",
    type=click.Path(file_okay=True, dir_okay=False, resolve_path=True, path_type=Path),
    help=(
        "Specify the filename containing the Ingress' TLS configuration as a YAML list. "
        "It's your responsibility to make sure that it's a valid Kubernetes config fragment. "
    ),
)
@click.option(
    "--ingress-domain",
    default=DEFAULT_INGRESS_DOMAIN,
    help=(
        "Specify the top level domain to be used for the Ingress. "
        f"Default is {DEFAULT_INGRESS_DOMAIN}"
    ),
)
@click.option(
    "--deployment-annotations-file",
    type=click.Path(file_okay=True, dir_okay=False, resolve_path=True, path_type=Path),
    help=(
        "Specify the filename containing the extra Deployment's annotaions as a YAML object. "
        "It's your responsibility to make sure that it's a valid Kubernetes config fragment. "
    ),
)
@click.option(
    "--deployment-service-account-name", help="Service account name to be used for the Deployment."
)
def main(
    compose_file: Path,
    project_name: str,
    repository_name: str,
    branch_name: str,
    default_image: str,
    ingress_domain: str,
    extra_manifest: Sequence[Path],
    ingress_for_service: Optional[str] = None,
    ingress_tls_file: Optional[Path] = None,
    deployment_annotations_file: Optional[Path] = None,
    deployment_service_account_name: Optional[Path] = None,
) -> None:
    context = Context(
        docker_compose_path=compose_file,
        project_name=project_name,
        branch_name=branch_name,
        repository_name=repository_name,
        default_image=default_image,
        ingress_for_service=ingress_for_service,
        extra_manifest_paths=extra_manifest,
        ingress=IngressContext(domain=ingress_domain, tls_path=ingress_tls_file),
        deployment=DeploymentContext(
            annotations_path=deployment_annotations_file,
            service_account_name=deployment_service_account_name,
        ),
    )

    manifest_data = generate_manifest_from_docker_compose(context)

    output_raw_manifest(context, manifest_data)


if __name__ == "__main__":
    main()



from pathlib import Path
from typing import Optional, Union

from komposer.types.base import ImmutableBaseModel

EnvironmentMap = dict[str, Optional[str]]

EnvironmentArray = list[str]

Environment = Union[EnvironmentMap, EnvironmentArray]


class Service(ImmutableBaseModel):
    image: Optional[str] = None
    ports: list[str] = []
    command: Optional[Union[str, list[str]]] = None
    env_file: Optional[Path] = None
    environment: Optional[Environment] = None


Services = dict[str, Service]


class DockerCompose(ImmutableBaseModel):
    services: Services



import os

from envsubst import envsubst

from komposer.types.cli import Context


def _komposer_service_prefix_value(context: Context) -> str:
    return context.manifest_prefix


def _komposer_ingress_domain_value(context: Context) -> str:
    return context.ingress.domain


komposer_env_variables_map = {
    "KOMPOSER_SERVICE_PREFIX": _komposer_service_prefix_value,
    "KOMPOSER_INGRESS_DOMAIN": _komposer_ingress_domain_value,
}


def replace_komposer_env_variables(context: Context, manifest_str: str) -> str:
    # Store original values of the env variables
    original_environ = os.environ.copy()

    try:
        # Inject all the env variables
        for env_name, env_fn in komposer_env_variables_map.items():
            env_value = env_fn(context)
            os.environ[env_name] = env_value

        # Render the new manifest
        manifest_str = envsubst(manifest_str)

    finally:
        # Restore original environment
        os.environ.clear()
        os.environ.update(original_environ)

    return manifest_str



import itertools
from collections.abc import Mapping, Sequence
from typing import Any, Union

from komposer.exceptions import (
    ExtraManifestInvalidYamlError,
    ExtraManifestMissingMetadataError,
    ExtraManifestMissingNameError,
)
from komposer.types import kubernetes
from komposer.types.cli import Context
from komposer.utils import load_yaml


def update_item_metadata_labels(item: Mapping, labels: Mapping) -> None:
    item["metadata"].setdefault("labels", {}).update(labels)


def update_item_metadata_name(item: Mapping, manifest_prefix: str) -> None:
    item["metadata"]["name"] = f"{manifest_prefix}-{item['metadata']['name']}"


def update_item_env_configmapkeyref_name(item: Mapping, manifest_prefix: str) -> None:
    containers = item.get("spec", {}).get("template", {}).get("spec", {}).get("containers", [])
    envs = itertools.chain(*(container.get("env", []) for container in containers))
    config_map_key_refs = (env.get("valueFrom", {}).get("configMapKeyRef") for env in envs)
    config_map_key_refs = (
        config_map_key_ref for config_map_key_ref in config_map_key_refs if config_map_key_ref
    )

    for config_map_key_ref in config_map_key_refs:
        config_map_key_ref["name"] = f"{manifest_prefix}-{config_map_key_ref['name']}"


def get_items_from_extra_manifest(extra_manifest_raw: Union[Mapping, Sequence]) -> list[dict]:
    # Returns empty list if null
    if extra_manifest_raw is None:
        return []

    # Returns if it's a list
    if isinstance(extra_manifest_raw, list):
        return extra_manifest_raw

    # Returns if it's a object
    if isinstance(extra_manifest_raw, dict):
        if extra_manifest_raw.get("kind") == "List":
            return extra_manifest_raw.get("items", [])

        return [extra_manifest_raw]

    # Not uspported
    raise ExtraManifestInvalidYamlError(
        f"Unsupported extra manifest type: {type(extra_manifest_raw)}"
    )


def ensure_metadata_is_present(extra_manifest_items: Sequence[Mapping[str, Any]]) -> None:
    for extra_manifest_item in extra_manifest_items:
        metadata = extra_manifest_item.get("metadata")
        if metadata is None:
            raise ExtraManifestMissingMetadataError(
                f"Missing metadata in extra manifest item: {extra_manifest_item}"
            )

        if "name" not in metadata:
            raise ExtraManifestMissingNameError(
                f"Missing metadata.name in extra manifest item: {extra_manifest_item}"
            )


def load_extra_manifests(context: Context) -> list[dict]:
    # Skip if no extra manifest
    if context.extra_manifest_paths is None:
        return []

    # Load manifest
    extra_manifests_str = (
        extra_manifest_path.read_text() for extra_manifest_path in context.extra_manifest_paths
    )

    # Parse manifest
    extra_manifests_raw = (
        load_yaml(extra_manifest_str) for extra_manifest_str in extra_manifests_str
    )

    # Just validate that the format of the file
    extra_manifest_items = list(
        itertools.chain(
            *(
                get_items_from_extra_manifest(extra_manifest_raw)
                for extra_manifest_raw in extra_manifests_raw
            )
        )
    )

    ensure_metadata_is_present(extra_manifest_items)

    # Update metadata labels and names and env's configmap names
    labels = kubernetes.Metadata.labels_from_context(context)
    manifest_prefix = context.manifest_prefix

    for item in extra_manifest_items:
        update_item_metadata_labels(item, labels)
        update_item_metadata_name(item, manifest_prefix)
        update_item_env_configmapkeyref_name(item, manifest_prefix)

    return extra_manifest_items



import re
from pathlib import Path
from typing import Any, Optional

from pydantic import validator

from komposer.types.base import ImmutableBaseModel
from komposer.utils import load_yaml

lowercase_kebak_re = re.compile(r"^[a-z0-9][a-z\-0-9]*[a-z0-9]$")


def ensure_lowercase_kebab(string: str) -> None:
    if not lowercase_kebak_re.match(string):
        raise ValueError("Not a lowercase kebab string")


class DeploymentContext(ImmutableBaseModel):
    annotations_path: Optional[Path] = None
    service_account_name: Optional[str] = None

    @property
    def annotations(self) -> Optional[Any]:
        return None if self.annotations_path is None else load_yaml(self.annotations_path)


class IngressContext(ImmutableBaseModel):
    tls_path: Optional[Path] = None
    domain: str

    @property
    def tls(self) -> Optional[Any]:
        return None if self.tls_path is None else load_yaml(self.tls_path)


class Context(ImmutableBaseModel):
    docker_compose_path: Path
    branch_name: str
    repository_name: str
    default_image: str
    project_name: Optional[str] = None
    ingress_for_service: Optional[str] = None
    extra_manifest_paths: list[Path]
    deployment: DeploymentContext
    ingress: IngressContext

    @validator("project_name")
    def project_name_matches_kubernetes_name(cls, value: Optional[str]) -> Optional[str]:
        if value is not None:
            ensure_lowercase_kebab(value)

        return value

    @validator("branch_name")
    def branch_name_matches_kubernetes_name(cls, value: str) -> str:
        ensure_lowercase_kebab(value)

        return value

    @validator("repository_name")
    def repository_name_matches_kubernetes_name(cls, value: str) -> str:
        ensure_lowercase_kebab(value)

        return value

    @property
    def manifest_prefix(self) -> str:
        fragments = [self.project_name, self.repository_name, self.branch_name]
        fragments_filtered = [fragment for fragment in fragments if fragment is not None]

        manifest_prefix = "-".join(fragments_filtered)

        return manifest_prefix



from komposer.core.container import generate_containers
from komposer.types import docker_compose, kubernetes
from komposer.types.cli import Context

LOCALHOST = "127.0.0.1"


def generate_host_aliases(services: docker_compose.Services) -> list[kubernetes.HostAlias]:
    if not services:
        return []

    host_names = [service_name for service_name in services.keys()]
    host_aliases = kubernetes.HostAlias(hostnames=host_names, ip=LOCALHOST)

    return [host_aliases]


def generate_deployment(
    context: Context, services: docker_compose.Services
) -> kubernetes.Deployment:
    host_aliases = generate_host_aliases(services)
    containers = generate_containers(context, services)
    metadata = kubernetes.Metadata.from_context_with_name(context, context.deployment.annotations)

    deployment = kubernetes.Deployment(
        metadata=metadata,
        spec=kubernetes.DeploymentSpec(
            selector=kubernetes.Selector(matchLabels=dict(metadata.labels)),
            template=kubernetes.Template(
                metadata=kubernetes.UnnamedMetadata(labels=dict(metadata.labels)),
                spec=kubernetes.TemplateSpec(
                    hostAliases=host_aliases,
                    containers=containers,
                    serviceAccountName=context.deployment.service_account_name,
                ),
            ),
        ),
    )

    return deployment



import json
import shlex
from pathlib import Path
from typing import Any, Optional, Union, cast

import stringcase
import yaml
from dotenv import dotenv_values
from pydantic import BaseModel

from komposer.types import docker_compose


def str_presenter(dumper: yaml.representer.SafeRepresenter, data: Any) -> yaml.nodes.Node:
    if len(data.splitlines()) > 1:  # check for multiline string
        return dumper.represent_scalar("tag:yaml.org,2002:str", data, style="|")
    return dumper.represent_scalar("tag:yaml.org,2002:str", data)


yaml.representer.SafeRepresenter.add_representer(str, str_presenter)


def to_kubernetes_name(string: str) -> str:
    name: str = stringcase.spinalcase(string.strip().lower().replace("/", "-").replace(":", "-"))

    if name.startswith("-"):
        name = name[1:]

    return name


def load_yaml(source: Union[Path, str]) -> Any:
    if isinstance(source, Path):
        source = source.read_text()

    content = yaml.safe_load(source)

    return content


def dump_yaml(data: Any) -> str:
    output: str = yaml.safe_dump(data)

    return output


def parse_docker_compose_file(compose_path: Path) -> docker_compose.DockerCompose:
    content = load_yaml(compose_path)
    config = docker_compose.DockerCompose.parse_obj(content)

    return config


def parse_env_file(env_file: Path) -> dict[str, Optional[str]]:
    environment = dotenv_values(env_file)

    return environment


def command_to_args(command: Optional[Union[str, list[str]]]) -> Optional[list[str]]:
    if isinstance(command, str):
        return shlex.split(command)

    return command


def as_json_object(type_: BaseModel) -> dict[str, Any]:
    return cast(dict[str, Any], json.loads(type_.json(by_alias=True)))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['komposer', 'komposer.core', 'komposer.types']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'click>=8.1.3,<9.0.0',
 'envsubst>=0.1.5,<0.2.0',
 'pydantic>=1.9.1,<2.0.0',
 'python-dotenv>=0.20',
 'stringcase>=1.2.0,<2.0.0']

entry_points = \
{'console_scripts': ['komposer = komposer.cli:main']}

setup_kwargs = {
    'name': 'komposer',
    'version': '0.1.8',
    'description': 'Tool to convert a Docker Compose file into a Kubernetes manifest',
    'long_description': '# komposer\n\n![main build status](https://github.com/expobrain/komposer/actions/workflows/main.yml/badge.svg?branch=main)\n\nKomposer is a CLI tool to convert a Docker Compose file into a Kubernetes manifest file so that you can deploy your Docker Compose stack it into a single Kubernetes Pod.\n\n# Documentation\n\nSee the official [documentation](https://expobrain.github.io/komposer/) for details information about the CLI usage.\n\n## To-do\n\n- set ingress annotations from CLI as file\n- set ingress paths from CLI as a file\n- able to select the Ingress class name\n- able to set custom resource limits\n- add annotations to all Kubernetes items with Komposer version\n- use labels in Docker Compose file as alternative for CLI options\n',
    'author': 'Daniele Esposti',
    'author_email': 'daniele.esposti@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/expobrain/komposer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from collections.abc import Iterable
from io import StringIO
from pathlib import Path
from typing import Optional

from dotenv import dotenv_values

from komposer.types import docker_compose, kubernetes
from komposer.types.cli import Context
from komposer.utils import parse_env_file, to_kubernetes_name


def _generate_config_map_from_environment(
    context: Context, service_name: str, environment: docker_compose.Environment
) -> kubernetes.ConfigMap:
    if isinstance(environment, list):
        stream = StringIO("\n".join(environment))
        environment = dotenv_values(stream=stream)

    name = to_kubernetes_name(service_name)

    config_map = kubernetes.ConfigMap(
        metadata=kubernetes.Metadata.from_context_with_suffix(context, name), data=environment
    )

    return config_map


def _generate_config_map_from_env_file(
    context: Context, env_file: Path
) -> Optional[kubernetes.ConfigMap]:
    env_file_full_path = (context.docker_compose_path.parent / env_file).resolve()
    env_vars = parse_env_file(env_file_full_path)

    if not env_vars:
        return None

    name = to_kubernetes_name(env_file.name)

    config_map = kubernetes.ConfigMap(
        metadata=kubernetes.Metadata.from_context_with_suffix(context, name), data=env_vars
    )

    return config_map


def generate_config_maps(
    context: Context, compose: docker_compose.DockerCompose
) -> Iterable[kubernetes.ConfigMap]:
    config_maps = {}

    for service_name, service in compose.services.items():
        if service.env_file:
            config_map = _generate_config_map_from_env_file(context, service.env_file)
        elif service.environment:
            config_map = _generate_config_map_from_environment(
                context, service_name, service.environment
            )
        else:
            config_map = None

        if config_map:
            config_maps[config_map.metadata.name] = config_map

    return config_maps.values()
